@@ -1,19 +1,24 @@
 from __future__ import annotations
 
 import asyncio
+from datetime import datetime, timezone
+import json
 import logging
-import sqlite3
+import re
+import types
 from typing import Any
 
 import aiosqlite
 
 import zigpy.appdb_schemas
+import zigpy.backups
 import zigpy.device
 import zigpy.endpoint
 import zigpy.group
 import zigpy.neighbor
 import zigpy.profiles
 import zigpy.quirks
+import zigpy.state
 import zigpy.types as t
 import zigpy.typing
 import zigpy.util
@@ -22,21 +27,72 @@ from zigpy.zdo import types as zdo_t
 
 LOGGER = logging.getLogger(__name__)
 
-DB_VERSION = 7
+DB_VERSION = 10
 DB_V = f"_v{DB_VERSION}"
+MIN_SQLITE_VERSION = (3, 24, 0)
+
+UNIX_EPOCH = datetime.fromtimestamp(0, tz=timezone.utc)
+DB_V_REGEX = re.compile(r"(?:_v\d+)?$")
+
+
+def _import_compatible_sqlite3(min_version: tuple[int, int, int]) -> types.ModuleType:
+    """
+    Loads an SQLite module with a library version matching the provided constraint.
+    """
+
+    import sqlite3
+
+    try:
+        import pysqlite3
+    except ImportError:
+        pysqlite3 = None
+
+    for module in [sqlite3, pysqlite3]:
+        if module is None:
+            continue
+
+        LOGGER.debug("SQLite version for %s: %s", module, module.sqlite_version)
+
+        if module.sqlite_version_info >= min_version:
+            return module
+    else:
+        min_ver = ".".join(map(str, min_version))
+
+        raise RuntimeError(
+            f"zigpy requires SQLite {min_ver} or newer. If your distribution does not"
+            f" provide a more recent release, install pysqlite3 with"
+            f" `pip install pysqlite3-binary`"
+        )
+
+
+sqlite3 = _import_compatible_sqlite3(min_version=MIN_SQLITE_VERSION)
 
 
 def _register_sqlite_adapters():
     def adapt_ieee(eui64):
         return str(eui64)
 
-    aiosqlite.register_adapter(t.EUI64, adapt_ieee)
-    aiosqlite.register_adapter(t.ExtendedPanId, adapt_ieee)
+    sqlite3.register_adapter(t.EUI64, adapt_ieee)
+    sqlite3.register_adapter(t.ExtendedPanId, adapt_ieee)
 
     def convert_ieee(s):
         return t.EUI64.convert(s.decode())
 
-    aiosqlite.register_converter("ieee", convert_ieee)
+    sqlite3.register_converter("ieee", convert_ieee)
+
+
+def aiosqlite_connect(
+    database: str, iter_chunk_size: int = 64, **kwargs
+) -> aiosqlite.Connection:
+    """
+    Copy of the the `aiosqlite.connect` function that connects using either the built-in
+    `sqlite3` module or the imported `pysqlite3` module.
+    """
+
+    return aiosqlite.Connection(
+        connector=lambda: sqlite3.connect(str(database), **kwargs),
+        iter_chunk_size=iter_chunk_size,
+    )
 
 
 def decode_str_attribute(value: str | bytes) -> str:
@@ -56,7 +112,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
 
         self._db = connection
         self._application = application
-        self._callback_handlers = asyncio.Queue()
+        self._callback_handlers: asyncio.Queue = asyncio.Queue()
         self.running = False
         self._worker_task = asyncio.create_task(self._worker())
 
@@ -68,6 +124,11 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             if status != "ok":
                 LOGGER.error("SQLite database file is corrupted!\n%s", status)
 
+        # Truncate the SQLite journal file instead of deleting it after transactions
+        await self._set_isolation_level(None)
+        await self.execute("PRAGMA journal_mode = TRUNCATE")
+        await self._set_isolation_level("DEFERRED")
+
         await self.execute("PRAGMA foreign_keys = ON")
         await self._run_migrations()
 
@@ -76,15 +137,15 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         cls, database_file: str, app: zigpy.typing.ControllerApplicationType
     ) -> PersistingListener:
         """Create an instance of persisting listener."""
-        sqlite_conn = await aiosqlite.connect(
-            database_file, detect_types=sqlite3.PARSE_DECLTYPES
+        sqlite_conn = await aiosqlite_connect(
+            database_file,
+            detect_types=sqlite3.PARSE_DECLTYPES,
+            isolation_level="DEFERRED",  # The default is "", an alias for "DEFERRED"
         )
         listener = cls(sqlite_conn, app)
 
         try:
             await listener.initialize_tables()
-        except asyncio.CancelledError:
-            raise
         except Exception:
             await listener.shutdown()
             raise
@@ -100,15 +161,13 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             assert handler
             try:
                 await handler(*args)
-            except aiosqlite.Error as exc:
+            except sqlite3.Error as exc:
                 LOGGER.debug(
                     "Error handling '%s' event with %s params: %s",
                     cb_name,
                     args,
                     str(exc),
                 )
-            except asyncio.CancelledError:
-                raise
             except Exception as ex:
                 LOGGER.error(
                     "Unexpected error while processing %s(%s): %s", cb_name, args, ex
@@ -121,6 +180,12 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         await self._callback_handlers.join()
         if not self._worker_task.done():
             self._worker_task.cancel()
+
+        # Delete the journal on shutdown
+        await self._set_isolation_level(None)
+        await self.execute("PRAGMA journal_mode = DELETE")
+        await self._set_isolation_level("DEFERRED")
+
         await self._db.close()
 
     def enqueue(self, cb_name: str, *args) -> None:
@@ -130,6 +195,10 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             return
         self._callback_handlers.put_nowait((cb_name, args))
 
+    async def _set_isolation_level(self, level: str | None):
+        """Set the SQLite statement isolation level in a thread-safe way."""
+        await self._db._execute(lambda: setattr(self._db, "isolation_level", level))
+
     def execute(self, *args, **kwargs):
         return self._db.execute(*args, **kwargs)
 
@@ -157,6 +226,19 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     def device_left(self, device: zigpy.typing.DeviceType) -> None:
         pass
 
+    def device_last_seen_updated(
+        self, device: zigpy.typing.DeviceType, last_seen: datetime
+    ) -> None:
+        """Device last_seen time is updated."""
+        self.enqueue("_save_device_last_seen", device.ieee, last_seen)
+
+    async def _save_device_last_seen(self, ieee: t.EUI64, last_seen: datetime) -> None:
+        await self.execute(
+            f"UPDATE devices{DB_V} SET last_seen=? WHERE ieee=?",
+            (last_seen.timestamp(), ieee),
+        )
+        await self._db.commit()
+
     def device_relays_updated(
         self, device: zigpy.typing.DeviceType, relays: t.Relays | None
     ) -> None:
@@ -167,7 +249,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         if relays is None:
             await self.execute(f"DELETE FROM relays{DB_V} WHERE ieee = ?", (ieee,))
         else:
-            q = f"INSERT OR REPLACE INTO relays{DB_V} VALUES (?, ?)"
+            q = f"""INSERT INTO relays{DB_V} VALUES (?, ?)
+                        ON CONFLICT (ieee)
+                        DO UPDATE SET relays=excluded.relays"""
             await self.execute(q, (ieee, relays.serialize()))
 
         await self._db.commit()
@@ -204,7 +288,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     async def _unsupported_attribute_added(
         self, ieee: t.EUI64, endpoint_id: int, cluster_id: int, attrid: int
     ) -> None:
-        q = f"INSERT OR REPLACE INTO unsupported_attributes{DB_V} VALUES (?, ?, ?, ?)"
+        q = f"""INSERT INTO unsupported_attributes{DB_V} VALUES (?, ?, ?, ?)
+                   ON CONFLICT (ieee, endpoint_id, cluster, attrid)
+                   DO NOTHING"""
         await self.execute(q, (ieee, endpoint_id, cluster_id, attrid))
         await self._db.commit()
 
@@ -220,8 +306,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         rows = [(neighbors.ieee,) + n.neighbor.as_tuple() for n in neighbors.neighbors]
 
         await self._db.executemany(
-            f"INSERT INTO neighbors{DB_V} VALUES (?,?,?,?,?,?,?,?,?,?,?,?)",
-            rows,
+            f"INSERT INTO neighbors{DB_V} VALUES (?,?,?,?,?,?,?,?,?,?,?,?)", rows
         )
         await self._db.commit()
 
@@ -230,7 +315,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         self.enqueue("_group_added", group)
 
     async def _group_added(self, group: zigpy.group.Group) -> None:
-        q = f"INSERT OR REPLACE INTO groups{DB_V} VALUES (?, ?)"
+        q = f"""INSERT INTO groups{DB_V} VALUES (?, ?)
+                    ON CONFLICT (group_id)
+                    DO UPDATE SET name=excluded.name"""
         await self.execute(q, (group.group_id, group.name))
         await self._db.commit()
 
@@ -243,7 +330,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     async def _group_member_added(
         self, group: zigpy.group.Group, ep: zigpy.typing.EndpointType
     ) -> None:
-        q = f"INSERT OR REPLACE INTO group_members{DB_V} VALUES (?, ?, ?)"
+        q = f"""INSERT INTO group_members{DB_V} VALUES (?, ?, ?)
+                    ON CONFLICT
+                    DO NOTHING"""
         await self.execute(q, (group.group_id, *ep.unique_id))
         await self._db.commit()
 
@@ -282,13 +371,22 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         self.enqueue("_save_device", device)
 
     async def _save_device(self, device: zigpy.typing.DeviceType) -> None:
-        try:
-            q = f"INSERT INTO devices{DB_V} (ieee, nwk, status) VALUES (?, ?, ?)"
-            await self.execute(q, (device.ieee, device.nwk, device.status))
-        except aiosqlite.IntegrityError:
-            LOGGER.debug("Device %s already exists. Updating it.", device.ieee)
-            q = f"UPDATE devices{DB_V} SET nwk=?, status=? WHERE ieee=?"
-            await self.execute(q, (device.nwk, device.status, device.ieee))
+        q = f"""INSERT INTO devices{DB_V} (ieee, nwk, status, last_seen)
+                    VALUES (?, ?, ?, ?)
+                    ON CONFLICT (ieee)
+                    DO UPDATE SET
+                        nwk=excluded.nwk,
+                        status=excluded.status,
+                        last_seen=excluded.last_seen"""
+        await self.execute(
+            q,
+            (
+                device.ieee,
+                device.nwk,
+                device.status,
+                (device._last_seen or UNIX_EPOCH).timestamp(),
+            ),
+        )
 
         if device.node_desc is not None:
             await self._save_node_descriptor(device)
@@ -306,33 +404,55 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         await self._db.commit()
 
     async def _save_endpoints(self, device: zigpy.typing.DeviceType) -> None:
-        endpoints = []
-        for ep in device.non_zdo_endpoints:
-            eprow = (
+        rows = [
+            (
                 device.ieee,
                 ep.endpoint_id,
                 ep.profile_id,
                 ep.device_type,
                 ep.status,
             )
-            endpoints.append(eprow)
+            for ep in device.non_zdo_endpoints
+        ]
 
-        q = f"INSERT OR REPLACE INTO endpoints{DB_V} VALUES (?, ?, ?, ?, ?)"
-        await self._db.executemany(q, endpoints)
+        q = f"""INSERT INTO endpoints{DB_V} VALUES (?, ?, ?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id)
+                    DO UPDATE SET
+                        profile_id=excluded.profile_id,
+                        device_type=excluded.device_type,
+                        status=excluded.status"""
+
+        await self._db.executemany(q, rows)
 
     async def _save_node_descriptor(self, device: zigpy.typing.DeviceType) -> None:
-        await self.execute(
-            f"INSERT OR REPLACE INTO node_descriptors{DB_V}"
-            f" VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
-            (device.ieee,) + device.node_desc.as_tuple(),
-        )
+        q = f"""INSERT INTO node_descriptors{DB_V}
+                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
+                    ON CONFLICT (ieee)
+                    DO UPDATE SET
+                logical_type=excluded.logical_type,
+                complex_descriptor_available=excluded.complex_descriptor_available,
+                user_descriptor_available=excluded.user_descriptor_available,
+                reserved=excluded.reserved,
+                aps_flags=excluded.aps_flags,
+                frequency_band=excluded.frequency_band,
+                mac_capability_flags=excluded.mac_capability_flags,
+                manufacturer_code=excluded.manufacturer_code,
+                maximum_buffer_size=excluded.maximum_buffer_size,
+                maximum_incoming_transfer_size=excluded.maximum_incoming_transfer_size,
+                server_mask=excluded.server_mask,
+                maximum_outgoing_transfer_size=excluded.maximum_outgoing_transfer_size,
+                descriptor_capability_field=excluded.descriptor_capability_field"""
+
+        await self.execute(q, (device.ieee,) + device.node_desc.as_tuple())
 
     async def _save_input_clusters(self, endpoint: zigpy.typing.EndpointType) -> None:
         clusters = [
             (endpoint.device.ieee, endpoint.endpoint_id, cluster.cluster_id)
             for cluster in endpoint.in_clusters.values()
         ]
-        q = f"INSERT OR REPLACE INTO in_clusters{DB_V} VALUES (?, ?, ?)"
+        q = f"""INSERT INTO in_clusters{DB_V} VALUES (?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id, cluster)
+                    DO NOTHING"""
         await self._db.executemany(q, clusters)
 
     async def _save_attribute_cache(self, ep: zigpy.typing.EndpointType) -> None:
@@ -341,7 +461,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             for cluster in ep.in_clusters.values()
             for attrid, value in cluster._attr_cache.items()
         ]
-        q = f"INSERT OR REPLACE INTO attributes_cache{DB_V} VALUES (?, ?, ?, ?, ?)"
+        q = f"""INSERT INTO attributes_cache{DB_V} VALUES (?, ?, ?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id, cluster, attrid)
+                    DO UPDATE SET value=excluded.value"""
         await self._db.executemany(q, clusters)
 
     async def _save_unsupported_attributes(self, ep: zigpy.typing.EndpointType) -> None:
@@ -351,7 +473,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             for attr in cluster.unsupported_attributes
             if isinstance(attr, int)
         ]
-        q = f"INSERT OR REPLACE INTO unsupported_attributes{DB_V} VALUES (?, ?, ?, ?)"
+        q = f"""INSERT INTO unsupported_attributes{DB_V} VALUES (?, ?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id, cluster, attrid)
+                    DO NOTHING"""
         await self._db.executemany(q, clusters)
 
     async def _save_output_clusters(self, endpoint: zigpy.typing.EndpointType) -> None:
@@ -359,16 +483,42 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             (endpoint.device.ieee, endpoint.endpoint_id, cluster.cluster_id)
             for cluster in endpoint.out_clusters.values()
         ]
-        q = f"INSERT OR REPLACE INTO out_clusters{DB_V} VALUES (?, ?, ?)"
+        q = f"""INSERT INTO out_clusters{DB_V} VALUES (?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id, cluster)
+                    DO NOTHING"""
         await self._db.executemany(q, clusters)
 
     async def _save_attribute(
         self, ieee: t.EUI64, endpoint_id: int, cluster_id: int, attrid: int, value: Any
     ) -> None:
-        q = f"INSERT OR REPLACE INTO attributes_cache{DB_V} VALUES (?, ?, ?, ?, ?)"
+        q = f"""INSERT INTO attributes_cache{DB_V} VALUES (?, ?, ?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id, cluster, attrid)
+                    DO UPDATE SET
+                        value=excluded.value"""
         await self.execute(q, (ieee, endpoint_id, cluster_id, attrid, value))
         await self._db.commit()
 
+    def network_backup_created(self, backup: zigpy.backups.NetworkBackup) -> None:
+        self.enqueue("_network_backup_created", json.dumps(backup.as_dict()))
+
+    async def _network_backup_created(self, backup_json: str) -> None:
+        q = f"""INSERT INTO network_backups{DB_V} VALUES (?, ?)
+                    ON CONFLICT (id)
+                    DO UPDATE SET
+                        backup_json=excluded.backup_json"""
+
+        await self.execute(q, (None, backup_json))
+        await self._db.commit()
+
+    def network_backup_removed(self, backup: zigpy.backups.NetworkBackup) -> None:
+        self.enqueue("_network_backup_removed", json.dumps(backup.as_dict()))
+
+    async def _network_backup_removed(self, backup_json: str) -> None:
+        q = f"""DELETE FROM network_backups{DB_V} WHERE backup_json=?"""
+
+        await self.execute(q, (backup_json,))
+        await self._db.commit()
+
     async def load(self) -> None:
         LOGGER.debug("Loading application state")
         await self._load_devices()
@@ -389,6 +539,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         await self._load_group_members()
         await self._load_relays()
         await self._load_neighbors()
+        await self._load_network_backups()
         await self._register_device_listeners()
 
     async def _load_attributes(self, filter: str = None) -> None:
@@ -422,9 +573,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 )
 
                 # Populate the device's manufacturer and model attributes
-                if cluster == Basic.cluster_id and attrid == 4:
+                if cluster == Basic.cluster_id and attrid == 0x0004:
                     dev.manufacturer = decode_str_attribute(value)
-                elif cluster == Basic.cluster_id and attrid == 5:
+                elif cluster == Basic.cluster_id and attrid == 0x0005:
                     dev.model = decode_str_attribute(value)
 
     async def _load_unsupported_attributes(self) -> None:
@@ -446,10 +597,13 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
 
     async def _load_devices(self) -> None:
         async with self.execute(f"SELECT * FROM devices{DB_V}") as cursor:
-            async for (ieee, nwk, status) in cursor:
+            async for (ieee, nwk, status, last_seen) in cursor:
                 dev = self._application.add_device(ieee, nwk)
                 dev.status = zigpy.device.Status(status)
 
+                if last_seen > 0:
+                    dev.last_seen = last_seen
+
     async def _load_node_descriptors(self) -> None:
         async with self.execute(f"SELECT * FROM node_descriptors{DB_V}") as cursor:
             async for (ieee, *fields) in cursor:
@@ -495,10 +649,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             async for (group_id, ieee, ep_id) in cursor:
                 dev = self._application.get_device(ieee)
                 group = self._application.groups[group_id]
-                group.add_member(
-                    dev.endpoints[ep_id],
-                    suppress_event=True,
-                )
+                group.add_member(dev.endpoints[ep_id], suppress_event=True)
 
     async def _load_relays(self) -> None:
         async with self.execute(f"SELECT * FROM relays{DB_V}") as cursor:
@@ -514,19 +665,42 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 assert neighbor.is_valid
                 dev.neighbors.add_neighbor(neighbor)
 
+    async def _load_network_backups(self) -> None:
+        self._application.backups.backups.clear()
+
+        async with self.execute(
+            f"SELECT * FROM network_backups{DB_V} ORDER BY id"
+        ) as cursor:
+            async for id_, backup_json in cursor:
+                backup = zigpy.backups.NetworkBackup.from_dict(json.loads(backup_json))
+                self._application.backups.add_backup(backup)
+
     async def _register_device_listeners(self) -> None:
         for dev in self._application.devices.values():
             dev.add_context_listener(self)
             dev.neighbors.add_context_listener(self)
 
-    async def _table_exists(self, name: str) -> bool:
-        async with self.execute(
-            "SELECT count(*) FROM sqlite_master WHERE type='table' AND name=?",
-            [name],
-        ) as cursor:
-            (count,) = await cursor.fetchone()
+    async def _get_table_versions(self) -> dict[str, str]:
+        tables = {}
 
-        return bool(count)
+        async with self.execute(
+            "SELECT name FROM sqlite_master WHERE type='table'"
+        ) as cursor:
+            async for (name,) in cursor:
+                # Ignore tables internal to SQLite
+                if name.startswith("sqlite_"):
+                    continue
+
+                # The regex will always return a match
+                match = DB_V_REGEX.search(name)
+                assert match is not None
+
+                tables[name] = match.group(0)
+
+        return tables
+
+    async def _table_exists(self, name: str) -> bool:
+        return name in (await self._get_table_versions())
 
     async def _run_migrations(self):
         """Migrates the database to the newest schema."""
@@ -560,6 +734,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 (self._migrate_to_v5, 5),
                 (self._migrate_to_v6, 6),
                 (self._migrate_to_v7, 7),
+                (self._migrate_to_v8, 8),
+                (self._migrate_to_v9, 9),
+                (self._migrate_to_v10, 10),
             ]:
                 if db_version >= min(to_db_version, DB_VERSION):
                     continue
@@ -582,9 +759,27 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     ):
         """Copy rows from one set of tables into another."""
 
+        # Extract the "old" table version suffix
+        tables = await self._get_table_versions()
+        old_table_name = list(table_map.keys())[0]
+        old_version = tables[old_table_name]
+
+        # Check which tables would not be migrated
+        old_tables = [t for t, v in tables.items() if v == old_version]
+        unmigrated_old_tables = [t for t in old_tables if t not in table_map]
+
+        if unmigrated_old_tables:
+            raise RuntimeError(
+                f"The following tables were not migrated: {unmigrated_old_tables}"
+            )
+
         # Insertion order matters for foreign key constraints but any rows that fail
         # to insert due to constraint violations can be discarded
         for old_table, new_table in table_map.items():
+            # Ignore tables without a migration
+            if new_table is None:
+                continue
+
             async with self.execute(f"SELECT * FROM {old_table}") as cursor:
                 async for row in cursor:
                     placeholders = ",".join("?" * len(row))
@@ -593,7 +788,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                         await self.execute(
                             f"INSERT INTO {new_table} VALUES ({placeholders})", row
                         )
-                    except aiosqlite.IntegrityError as e:
+                    except sqlite3.IntegrityError as e:
                         if errors == "raise":
                             raise
                         elif errors == "warn":
@@ -660,6 +855,10 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 # These were migrated in v4
                 "neighbors_v4": "neighbors_v5",
                 "node_descriptors_v4": "node_descriptors_v5",
+                # Explicitly specify which tables will not be migrated
+                "devices": None,
+                "neighbors": None,
+                "node_descriptors": None,
             },
             errors="warn",
         )
@@ -680,6 +879,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 "attributes_cache_v5": "attributes_cache_v6",
                 "neighbors_v5": "neighbors_v6",
                 "node_descriptors_v5": "node_descriptors_v6",
+                "devices_v5": None,
             }
         )
 
@@ -698,11 +898,23 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 )
 
                 await self._migrate_tables(
-                    {"attributes": "attributes_cache_v6"}, errors="ignore"
+                    {
+                        "attributes": "attributes_cache_v6",
+                        "devices": None,
+                        "endpoints": None,
+                        "clusters": None,
+                        "neighbors": None,
+                        "node_descriptors": None,
+                        "output_clusters": None,
+                        "groups": None,
+                        "group_members": None,
+                        "relays": None,
+                    },
+                    errors="ignore",
                 )
 
     async def _migrate_to_v7(self):
-        """Schema v7 just adds a new table."""
+        """Schema v7 added the `unsupported_attributes` table."""
 
         # Copy the devices table first, it should have no conflicts
         await self.execute("INSERT INTO devices_v7 SELECT * FROM devices_v6")
@@ -717,5 +929,79 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 "attributes_cache_v6": "attributes_cache_v7",
                 "neighbors_v6": "neighbors_v7",
                 "node_descriptors_v6": "node_descriptors_v7",
+                "devices_v6": None,
+            }
+        )
+
+    async def _migrate_to_v8(self):
+        """Schema v8 added the `devices_v8.last_seen` column."""
+
+        async with self.execute("SELECT * FROM devices_v7") as cursor:
+            async for (ieee, nwk, status) in cursor:
+                # Set the default `last_seen` to the unix epoch
+                await self.execute(
+                    "INSERT INTO devices_v8 VALUES (?, ?, ?, ?)",
+                    (ieee, nwk, status, 0),
+                )
+
+        # Copy the devices table first, it should have no conflicts
+        await self._migrate_tables(
+            {
+                "endpoints_v7": "endpoints_v8",
+                "in_clusters_v7": "in_clusters_v8",
+                "out_clusters_v7": "out_clusters_v8",
+                "groups_v7": "groups_v8",
+                "group_members_v7": "group_members_v8",
+                "relays_v7": "relays_v8",
+                "attributes_cache_v7": "attributes_cache_v8",
+                "neighbors_v7": "neighbors_v8",
+                "node_descriptors_v7": "node_descriptors_v8",
+                "unsupported_attributes_v7": "unsupported_attributes_v8",
+                "devices_v7": None,
+            }
+        )
+
+    async def _migrate_to_v9(self):
+        """Schema v9 changed the data type of the `devices_v8.last_seen` column."""
+
+        await self.execute(
+            """INSERT INTO devices_v9 (ieee, nwk, status, last_seen)
+            SELECT ieee, nwk, status, last_seen / 1000.0 FROM devices_v8"""
+        )
+
+        await self._migrate_tables(
+            {
+                "endpoints_v8": "endpoints_v9",
+                "in_clusters_v8": "in_clusters_v9",
+                "out_clusters_v8": "out_clusters_v9",
+                "groups_v8": "groups_v9",
+                "group_members_v8": "group_members_v9",
+                "relays_v8": "relays_v9",
+                "attributes_cache_v8": "attributes_cache_v9",
+                "neighbors_v8": "neighbors_v9",
+                "node_descriptors_v8": "node_descriptors_v9",
+                "unsupported_attributes_v8": "unsupported_attributes_v9",
+                "devices_v8": None,
+            }
+        )
+
+    async def _migrate_to_v10(self):
+        """Schema v10 added a new `network_backups_v10` table."""
+
+        await self.execute("INSERT INTO devices_v10 SELECT * FROM devices_v9")
+
+        await self._migrate_tables(
+            {
+                "endpoints_v9": "endpoints_v10",
+                "in_clusters_v9": "in_clusters_v10",
+                "out_clusters_v9": "out_clusters_v10",
+                "groups_v9": "groups_v10",
+                "group_members_v9": "group_members_v10",
+                "relays_v9": "relays_v10",
+                "attributes_cache_v9": "attributes_cache_v10",
+                "neighbors_v9": "neighbors_v10",
+                "node_descriptors_v9": "node_descriptors_v10",
+                "unsupported_attributes_v9": "unsupported_attributes_v10",
+                "devices_v9": None,
             }
         )
