@@ -3,6 +3,8 @@
 import voluptuous as vol
 
 from zigpy.config.defaults import (
+    CONF_NWK_BACKUP_ENABLED_DEFAULT,
+    CONF_NWK_BACKUP_PERIOD_DEFAULT,
     CONF_NWK_CHANNEL_DEFAULT,
     CONF_NWK_CHANNELS_DEFAULT,
     CONF_NWK_EXTENDED_PAN_ID_DEFAULT,
@@ -13,8 +15,10 @@ from zigpy.config.defaults import (
     CONF_NWK_TC_LINK_KEY_DEFAULT,
     CONF_NWK_UPDATE_ID_DEFAULT,
     CONF_OTA_IKEA_DEFAULT,
+    CONF_OTA_INOVELLI_DEFAULT,
     CONF_OTA_LEDVANCE_DEFAULT,
     CONF_OTA_OTAU_DIR_DEFAULT,
+    CONF_OTA_SALUS_DEFAULT,
     CONF_TOPO_SCAN_ENABLED_DEFAULT,
     CONF_TOPO_SCAN_PERIOD_DEFAULT,
     CONF_TOPO_SKIP_COORDINATOR_DEFAULT,
@@ -35,11 +39,15 @@ CONF_NWK_KEY_SEQ = "key_sequence_number"
 CONF_NWK_TC_ADDRESS = "tc_address"
 CONF_NWK_TC_LINK_KEY = "tc_link_key"
 CONF_NWK_UPDATE_ID = "update_id"
+CONF_NWK_BACKUP_ENABLED = "backup_enabled"
+CONF_NWK_BACKUP_PERIOD = "backup_period"
 CONF_OTA = "ota"
 CONF_OTA_DIR = "otau_directory"
 CONF_OTA_IKEA = "ikea_provider"
 CONF_OTA_IKEA_URL = "ikea_update_url"
+CONF_OTA_INOVELLI = "inovelli_provider"
 CONF_OTA_LEDVANCE = "ledvance_provider"
+CONF_OTA_SALUS = "salus_provider"
 CONF_TOPO_SCAN_PERIOD = "topology_scan_period"
 CONF_TOPO_SCAN_ENABLED = "topology_scan_enabled"
 CONF_TOPO_SKIP_COORDINATOR = "topology_scan_skip_coordinator"
@@ -79,7 +87,9 @@ SCHEMA_OTA = {
     vol.Optional(CONF_OTA_DIR, default=CONF_OTA_OTAU_DIR_DEFAULT): vol.Any(None, str),
     vol.Optional(CONF_OTA_IKEA, default=CONF_OTA_IKEA_DEFAULT): cv_boolean,
     vol.Optional(CONF_OTA_IKEA_URL): vol.Url(),
+    vol.Optional(CONF_OTA_INOVELLI, default=CONF_OTA_INOVELLI_DEFAULT): cv_boolean,
     vol.Optional(CONF_OTA_LEDVANCE, default=CONF_OTA_LEDVANCE_DEFAULT): cv_boolean,
+    vol.Optional(CONF_OTA_SALUS, default=CONF_OTA_SALUS_DEFAULT): cv_boolean,
 }
 
 ZIGPY_SCHEMA = vol.Schema(
@@ -96,6 +106,12 @@ ZIGPY_SCHEMA = vol.Schema(
         vol.Optional(
             CONF_TOPO_SKIP_COORDINATOR, default=CONF_TOPO_SKIP_COORDINATOR_DEFAULT
         ): cv_boolean,
+        vol.Optional(
+            CONF_NWK_BACKUP_ENABLED, default=CONF_NWK_BACKUP_ENABLED_DEFAULT
+        ): cv_boolean,
+        vol.Optional(
+            CONF_NWK_BACKUP_PERIOD, default=CONF_NWK_BACKUP_PERIOD_DEFAULT
+        ): vol.All(cv_hex, vol.Range(min=1)),
     },
     extra=vol.ALLOW_EXTRA,
 )
