@@ -5,7 +5,14 @@ from typing import Optional
 
 import attr
 
-from zigpy.config import CONF_OTA, CONF_OTA_DIR, CONF_OTA_IKEA, CONF_OTA_LEDVANCE
+from zigpy.config import (
+    CONF_OTA,
+    CONF_OTA_DIR,
+    CONF_OTA_IKEA,
+    CONF_OTA_INOVELLI,
+    CONF_OTA_LEDVANCE,
+    CONF_OTA_SALUS,
+)
 from zigpy.ota.image import BaseOTAImage, ImageKey, OTAImageHeader
 import zigpy.ota.provider
 from zigpy.ota.validators import check_invalid
@@ -94,7 +101,7 @@ class OTA(zigpy.util.ListenableMixin):
 
     def __init__(self, app: ControllerApplicationType, *args, **kwargs):
         self._app: ControllerApplicationType = app
-        self._image_cache = {}
+        self._image_cache: dict[ImageKey, CachedImage] = {}
         self._not_initialized = True
         self._listeners = {}
         ota_config = app.config[CONF_OTA]
@@ -102,15 +109,28 @@ class OTA(zigpy.util.ListenableMixin):
             self.add_listener(zigpy.ota.provider.FileStore())
         if ota_config[CONF_OTA_IKEA]:
             self.add_listener(zigpy.ota.provider.Trådfri())
+        if ota_config[CONF_OTA_INOVELLI]:
+            self.add_listener(zigpy.ota.provider.Inovelli())
         if ota_config[CONF_OTA_LEDVANCE]:
             self.add_listener(zigpy.ota.provider.Ledvance())
+        if ota_config[CONF_OTA_SALUS]:
+            self.add_listener(zigpy.ota.provider.Salus())
 
     async def initialize(self) -> None:
         await self.async_event("initialize_provider", self._app.config[CONF_OTA])
         self._not_initialized = False
 
-    async def get_ota_image(self, manufacturer_id, image_type) -> Optional[CachedImage]:
-        key = ImageKey(manufacturer_id, image_type)
+    async def get_ota_image(
+        self, manufacturer_id, image_type, model=None
+    ) -> Optional[CachedImage]:
+        if manufacturer_id in (
+            zigpy.ota.provider.Salus.MANUFACTURER_ID,
+            zigpy.ota.provider.Inovelli.MANUFACTURER_ID,
+        ):  # Salus/computime/Inovelli do not pass a useful image_type
+            # in the message from the device. So construct key based on model name.
+            key = ImageKey(manufacturer_id, model)
+        else:
+            key = ImageKey(manufacturer_id, image_type)
         if key in self._image_cache and not self._image_cache[key].expired:
             return self._image_cache[key]
 
