@@ -3,7 +3,6 @@
 from __future__ import annotations
 
 import logging
-import typing
 
 import attr
 
@@ -106,7 +105,7 @@ class OTAImageHeader(t.Struct):
         return ImageKey(self.manufacturer_id, self.image_type)
 
     @classmethod
-    def deserialize(cls, data) -> tuple:
+    def deserialize(cls, data) -> tuple[OTAImageHeader, bytes]:
         hdr, data = super().deserialize(data)
         if hdr.upgrade_file_id != cls.MAGIC_VALUE:
             raise ValueError(
@@ -118,9 +117,12 @@ class OTAImageHeader(t.Struct):
 
 class ElementTagId(t.enum16):
     UPGRADE_IMAGE = 0x0000
-    ECDSA_SIGNATURE = 0x0001
-    ECDSA_SIGNING_CERTIFICATE = 0x0002
+    ECDSA_SIGNATURE_CRYPTO_SUITE_1 = 0x0001
+    ECDSA_SIGNING_CERTIFICATE_CRYPTO_SUITE_1 = 0x0002
     IMAGE_INTEGRITY_CODE = 0x0003
+    PICTURE_DATA = 0x0004
+    ECDSA_SIGNATURE_CRYPTO_SUITE_2 = 0x0005
+    ECDSA_SIGNING_CERTIFICATE_CRYPTO_SUITE_2 = 0x0006
 
 
 class LVBytes32(t.LVBytes):
@@ -142,7 +144,7 @@ class BaseOTAImage:
     header: OTAImageHeader
 
     @classmethod
-    def deserialize(cls, data):
+    def deserialize(cls, data) -> tuple[BaseOTAImage, bytes]:
         raise NotImplementedError()  # pragma: no cover
 
     def serialize(self):
@@ -158,7 +160,7 @@ class OTAImage(t.Struct, BaseOTAImage):
     subelements: t.List[SubElement]
 
     @classmethod
-    def deserialize(cls, data):
+    def deserialize(cls, data) -> tuple[OTAImage, bytes]:
         hdr, data = OTAImageHeader.deserialize(data)
         elements_len = hdr.image_size - hdr.header_length
 
@@ -197,7 +199,7 @@ class HueSBLOTAImage(BaseOTAImage):
         return self.header.serialize() + self.data
 
     @classmethod
-    def deserialize(cls, data) -> typing.Tuple["HueSBLOTAImage", bytes]:
+    def deserialize(cls, data) -> tuple[HueSBLOTAImage, bytes]:
         header, remaining_data = OTAImageHeader.deserialize(data)
         firmware = remaining_data[: header.image_size - len(header.serialize())]
 
@@ -219,13 +221,17 @@ class HueSBLOTAImage(BaseOTAImage):
         return cls(header=header, data=firmware), data[header.image_size :]
 
 
-def parse_ota_image(data: bytes) -> typing.Tuple[BaseOTAImage, bytes]:
+def parse_ota_image(data: bytes) -> tuple[BaseOTAImage, bytes]:
     """
     Attempts to extract any known OTA image type from data. Does not validate firmware.
     """
 
-    # IKEA container needs to be unwrapped
-    if data.startswith(b"NGIS"):
+    if len(data) > 4 and int.from_bytes(data[0:4], "little") + 21 == len(data):
+        # Legrand OTA images are prefixed with their unwrapped size and include a 1 + 16
+        # byte suffix
+        return OTAImage.deserialize(data[4:-17])
+    elif data.startswith(b"NGIS"):
+        # IKEA container needs to be unwrapped
         if len(data) <= 24:
             raise ValueError(
                 f"Data too short to contain IKEA container header: {len(data)}"
