@@ -1,9 +1,12 @@
+from __future__ import annotations
+
 import enum
 import inspect
 import struct
-from typing import Callable, Tuple, TypeVar
+from typing import Callable, TypeVar
 
 CALLABLE_T = TypeVar("CALLABLE_T", bound=Callable)  # pylint: disable=invalid-name
+T = TypeVar("T")
 
 
 class Bits(list):
@@ -35,8 +38,8 @@ class Bits(list):
         return bytes(serialized_bytes)
 
     @classmethod
-    def deserialize(cls, data) -> Tuple["Bits", bytes]:
-        bits = []
+    def deserialize(cls, data) -> tuple[Bits, bytes]:
+        bits: list[int] = []
 
         for byte in data:
             bits.extend((byte >> i) & 1 for i in range(7, -1, -1))
@@ -58,7 +61,7 @@ class FixedIntType(int):
 
         n = super().__new__(cls, *args, **kwargs)
 
-        if not cls._signed and not 0 <= n <= 2 ** cls._bits - 1:
+        if not cls._signed and not 0 <= n <= 2**cls._bits - 1:
             raise ValueError(f"{int(n)} is not an unsigned {cls._bits} bit integer")
         elif (
             cls._signed and not -(2 ** (cls._bits - 1)) <= n <= 2 ** (cls._bits - 1) - 1
@@ -104,11 +107,15 @@ class FixedIntType(int):
         if "__new__" not in cls.__dict__:
             cls.__new__ = cls.__new__
 
+        # XXX: The enum module sabotages pickling using the same logic.
+        if "__reduce_ex__" not in cls.__dict__:
+            cls.__reduce_ex__ = cls.__reduce_ex__
+
     def bits(self) -> Bits:
         return Bits([(self >> n) & 0b1 for n in range(self._bits - 1, -1, -1)])
 
     @classmethod
-    def from_bits(cls, bits: Bits) -> Tuple["FixedIntType", Bits]:
+    def from_bits(cls, bits: Bits) -> tuple[FixedIntType, Bits]:
         if len(bits) < cls._bits:
             raise ValueError(f"Not enough bits to decode {cls}: {bits}")
 
@@ -119,7 +126,7 @@ class FixedIntType(int):
             n |= bit & 1
 
         if cls._signed and n >= 2 ** (cls._bits - 1):
-            n -= 2 ** cls._bits
+            n -= 2**cls._bits
 
         return cls(n), bits[: -cls._bits]
 
@@ -130,7 +137,7 @@ class FixedIntType(int):
         return self.to_bytes(self._bits // 8, "little", signed=self._signed)
 
     @classmethod
-    def deserialize(cls, data: bytes) -> Tuple["FixedIntType", bytes]:
+    def deserialize(cls, data: bytes) -> tuple[FixedIntType, bytes]:
         if cls._bits % 8 != 0:
             raise TypeError(f"Integer type with {cls._bits} bits is not byte aligned")
 
@@ -294,9 +301,21 @@ def enum_factory(int_type: CALLABLE_T, undefined: str = "undefined") -> CALLABLE
             new._value_ = value
             return new
 
+        def __format__(self, format_spec: str) -> str:
+            if format_spec:
+                # Allow formatting the integer enum value
+                return self._member_type_.__format__(self, format_spec)
+            else:
+                # Otherwise, format it as its string representation
+                return object.__format__(repr(self), format_spec)
+
     return _NewEnum
 
 
+class enum1(enum_factory(uint1_t)):  # noqa: N801
+    pass
+
+
 class enum2(enum_factory(uint2_t)):  # noqa: N801
     pass
 
@@ -399,7 +418,7 @@ class BaseFloat(float):
         cls._size = size_bits // 8
 
     @staticmethod
-    def _convert_format(*, src: "BaseFloat", dst: "BaseFloat", n: int) -> int:
+    def _convert_format(*, src: BaseFloat, dst: BaseFloat, n: int) -> int:
         """
         Converts an integer representing a float from one format into another. Note:
 
@@ -416,7 +435,7 @@ class BaseFloat(float):
         src_exp = src_biased_exp - 2 ** (src._exponent_bits - 1)
 
         if src_biased_exp == (1 << src._exponent_bits) - 1:
-            dst_biased_exp = 2 ** dst._exponent_bits - 1
+            dst_biased_exp = 2**dst._exponent_bits - 1
         elif src_biased_exp == 0:
             dst_biased_exp = 0
         else:
@@ -443,7 +462,7 @@ class BaseFloat(float):
         ).to_bytes(self._size, "little")
 
     @classmethod
-    def deserialize(cls, data: bytes) -> Tuple["BaseFloat", bytes]:
+    def deserialize(cls, data: bytes) -> tuple[BaseFloat, bytes]:
         if len(data) < cls._size:
             raise ValueError(f"Data is too short to contain {cls._size} bytes")
 
@@ -489,13 +508,40 @@ class LVBytes(bytes):
         return cls(s), data[cls._prefix_length + num_bytes :]
 
 
+def LimitedLVBytes(max_len):  # noqa: N802
+    class LimitedLVBytes(LVBytes):
+        _max_len = max_len
+
+        def serialize(self):
+            if len(self) > self._max_len:
+                raise ValueError(f"LVBytes is too long (>{self._max_len})")
+            return super().serialize()
+
+    return LimitedLVBytes
+
+
+class LVBytesSize2(LVBytes):
+    def serialize(self):
+        if len(self) != 2:
+            raise ValueError("LVBytes must be of size 2")
+        return super().serialize()
+
+    @classmethod
+    def deserialize(cls, data):
+        d, r = super().deserialize(data)
+
+        if len(d) != 2:
+            raise ValueError("LVBytes must be of size 2")
+        return d, r
+
+
 class LongOctetString(LVBytes):
     _prefix_length = 2
 
 
 class KwargTypeMeta(type):
     # So things like `LVList[NWK, t.uint8_t]` are singletons
-    _anonymous_classes = {}
+    _anonymous_classes = {}  # type:ignore[var-annotated]
 
     def __new__(metaclass, name, bases, namespaces, **kwargs):
         cls_kwarg_attrs = namespaces.get("_getitem_kwargs", {})
@@ -590,7 +636,7 @@ class List(list, metaclass=KwargTypeMeta):
         return b"".join([self._item_type(i).serialize() for i in self])
 
     @classmethod
-    def deserialize(cls, data: bytes) -> Tuple["LVList", bytes]:
+    def deserialize(cls: type[T], data: bytes) -> tuple[T, bytes]:
         assert cls._item_type is not None
 
         lst = cls()
@@ -614,7 +660,7 @@ class LVList(list, metaclass=KwargTypeMeta):
         )
 
     @classmethod
-    def deserialize(cls, data: bytes) -> Tuple["LVList", bytes]:
+    def deserialize(cls: type[T], data: bytes) -> tuple[T, bytes]:
         assert cls._item_type is not None
         length, data = cls._length_type.deserialize(data)
         r = cls()
@@ -641,7 +687,7 @@ class FixedList(list, metaclass=KwargTypeMeta):
         return b"".join([self._item_type(i).serialize() for i in self])
 
     @classmethod
-    def deserialize(cls, data: bytes) -> Tuple["FixedList", bytes]:
+    def deserialize(cls: type[T], data: bytes) -> tuple[T, bytes]:
         assert cls._item_type is not None
         r = cls()
         for i in range(cls._length):
@@ -653,7 +699,7 @@ class FixedList(list, metaclass=KwargTypeMeta):
 class CharacterString(str):
     _prefix_length = 1
 
-    def serialize(self):
+    def serialize(self) -> bytes:
         if len(self) >= pow(256, self._prefix_length) - 1:
             raise ValueError("String is too long")
         return len(self).to_bytes(
@@ -661,7 +707,7 @@ class CharacterString(str):
         ) + self.encode("utf8")
 
     @classmethod
-    def deserialize(cls, data):
+    def deserialize(cls: type[T], data: bytes) -> tuple[T, bytes]:
         if len(data) < cls._prefix_length:
             raise ValueError("Data is too short")
 
@@ -671,7 +717,11 @@ class CharacterString(str):
             raise ValueError("Data is too short")
 
         raw = data[cls._prefix_length : cls._prefix_length + length]
-        r = cls(raw.split(b"\x00")[0].decode("utf8", errors="replace"))
+        text = raw.split(b"\x00")[0].decode("utf8", errors="replace")
+
+        # FIXME: figure out how to get this working: `T` is not behaving as expected in
+        # the classmethod when it is not bound.
+        r = cls(text)  # type:ignore[call-arg]
         r.raw = raw
         return r, data[cls._prefix_length + length :]
 
@@ -684,9 +734,9 @@ def LimitedCharString(max_len):  # noqa: N802
     class LimitedCharString(CharacterString):
         _max_len = max_len
 
-        def serialize(self):
+        def serialize(self) -> bytes:
             if len(self) > self._max_len:
-                raise ValueError("String is too long")
+                raise ValueError(f"String is too long (>{self._max_len})")
             return super().serialize()
 
     return LimitedCharString
@@ -709,46 +759,30 @@ def Optional(optional_item_type):
 class data8(FixedList, item_type=uint8_t, length=1):
     """General data, Discrete, 8 bit."""
 
-    pass
-
 
 class data16(FixedList, item_type=uint8_t, length=2):
     """General data, Discrete, 16 bit."""
 
-    pass
-
 
 class data24(FixedList, item_type=uint8_t, length=3):
     """General data, Discrete, 24 bit."""
 
-    pass
-
 
 class data32(FixedList, item_type=uint8_t, length=4):
     """General data, Discrete, 32 bit."""
 
-    pass
-
 
 class data40(FixedList, item_type=uint8_t, length=5):
     """General data, Discrete, 40 bit."""
 
-    pass
-
 
 class data48(FixedList, item_type=uint8_t, length=6):
     """General data, Discrete, 48 bit."""
 
-    pass
-
 
 class data56(FixedList, item_type=uint8_t, length=7):
     """General data, Discrete, 56 bit."""
 
-    pass
-
 
 class data64(FixedList, item_type=uint8_t, length=8):
     """General data, Discrete, 64 bit."""
-
-    pass
