@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import abc
 import asyncio
 import functools
@@ -5,10 +7,13 @@ import inspect
 import logging
 import sys
 import traceback
-from typing import Any, Coroutine, Dict, Optional, Tuple, Type, Union
+import typing
+import warnings
 
-from Crypto.Cipher import AES
 from crccheck.crc import CrcX25
+from cryptography.hazmat.primitives.ciphers import Cipher
+from cryptography.hazmat.primitives.ciphers.algorithms import AES
+from cryptography.hazmat.primitives.ciphers.modes import ECB
 
 from zigpy.exceptions import ControllerException, ZigbeeException
 import zigpy.types as t
@@ -17,7 +22,9 @@ LOGGER = logging.getLogger(__name__)
 
 
 class ListenableMixin:
-    _listeners: Dict
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._listeners: dict[int, tuple[typing.Callable, bool]] = {}
 
     def _add_listener(self, listener, include_context):
         id_ = id(listener)
@@ -46,7 +53,12 @@ class ListenableMixin:
                 else:
                     result.append(method(*args))
             except Exception as e:
-                LOGGER.warning("Error calling listener.%s: %s", method_name, e)
+                LOGGER.warning(
+                    "Error calling listener %r with args %r: %r", method, args, e
+                )
+                LOGGER.debug(
+                    "Error calling listener %r with args %r", method, args, exc_info=e
+                )
         return result
 
     async def async_event(self, method_name, *args):
@@ -65,7 +77,15 @@ class ListenableMixin:
         results = []
         for result in await asyncio.gather(*tasks, return_exceptions=True):
             if isinstance(result, Exception):
-                LOGGER.warning("Error calling listener: %s", result)
+                LOGGER.warning(
+                    "Error calling listener %r with args %r: %r", method, args, result
+                )
+                LOGGER.debug(
+                    "Error calling listener %r with args %r",
+                    method,
+                    args,
+                    exc_info=result,
+                )
             else:
                 results.append(result)
         return results
@@ -145,36 +165,42 @@ def retryable(retry_exceptions, tries=1, delay=0.1):
 retryable_request = retryable((ZigbeeException, asyncio.TimeoutError))
 
 
-def aes_mmo_hash_update(length, result, data):
-    while len(data) >= AES.block_size:
+def aes_mmo_hash_update(length: int, result: bytes, data: bytes) -> tuple[int, bytes]:
+    block_size = AES.block_size // 8
+
+    while len(data) >= block_size:
+        block = bytes(data[:block_size])
+
         # Encrypt
-        aes = AES.new(bytes(result), AES.MODE_ECB)
-        result = bytearray(aes.encrypt(bytes(data[: AES.block_size])))
+        aes = Cipher(AES(bytes(result)), ECB()).encryptor()
+        result = bytearray(aes.update(block) + aes.finalize())
 
-        # XOR
-        for i in range(AES.block_size):
-            result[i] ^= bytes(data[: AES.block_size])[i]
+        # XOR plaintext into ciphertext
+        for i in range(block_size):
+            result[i] ^= block[i]
 
-        data = data[AES.block_size :]
-        length += AES.block_size
+        data = data[block_size:]
+        length += block_size
 
     return (length, result)
 
 
-def aes_mmo_hash(data):
+def aes_mmo_hash(data: bytes) -> t.KeyData:
+    block_size = AES.block_size // 8
+
     result_len = 0
     remaining_length = 0
     length = len(data)
-    result = bytearray([0] * AES.block_size)
-    temp = bytearray([0] * AES.block_size)
+    result = bytearray([0] * block_size)
+    temp = bytearray([0] * block_size)
 
     if data and length > 0:
-        remaining_length = length & (AES.block_size - 1)
-        if length >= AES.block_size:
+        remaining_length = length & (block_size - 1)
+        if length >= block_size:
             # Mask out the lower byte since hash update will hash
             # everything except the last piece, if the last piece
             # is less than 16 bytes.
-            hashed_length = length & ~(AES.block_size - 1)
+            hashed_length = length & ~(block_size - 1)
             (result_len, result) = aes_mmo_hash_update(result_len, result, data)
             data = data[hashed_length:]
 
@@ -188,25 +214,25 @@ def aes_mmo_hash(data):
 
     # If appending the bit string will push us beyond the 16-byte boundary
     # we must hash that block and append another 16-byte block.
-    if (AES.block_size - remaining_length) < 3:
+    if (block_size - remaining_length) < 3:
         (result_len, result) = aes_mmo_hash_update(result_len, result, temp)
 
         # Since this extra data is due to the concatenation,
         # we remove that length. We want the length of data only
         # and not the padding.
-        result_len -= AES.block_size
-        temp = bytearray([0] * AES.block_size)
+        result_len -= block_size
+        temp = bytearray([0] * block_size)
 
     bit_size = result_len * 8
-    temp[AES.block_size - 2] = (bit_size >> 8) & 0xFF
-    temp[AES.block_size - 1] = (bit_size) & 0xFF
+    temp[block_size - 2] = (bit_size >> 8) & 0xFF
+    temp[block_size - 1] = (bit_size) & 0xFF
 
     (result_len, result) = aes_mmo_hash_update(result_len, result, temp)
 
-    return t.KeyData([t.uint8_t(c) for c in result])
+    return t.KeyData(result)
 
 
-def convert_install_code(code):
+def convert_install_code(code: bytes) -> t.KeyData:
     if len(code) not in (8, 10, 14, 18):
         return None
 
@@ -226,7 +252,7 @@ class Request:
         """Init context manager for requests."""
         assert sequence not in pending
         self._pending = pending
-        self._result = asyncio.Future()
+        self._result: asyncio.Future = asyncio.Future()
         self._sequence = sequence
 
     @property
@@ -267,17 +293,17 @@ class CatchingTaskMixin(LocalLogMixin):
 
     def create_catching_task(
         self,
-        target: Coroutine,
-        exceptions: Optional[Union[Type[Exception], Tuple]] = None,
+        target: typing.Coroutine,
+        exceptions: type[Exception] | tuple | None = None,
     ) -> None:
         """Create a task."""
         asyncio.ensure_future(self.catching_coro(target, exceptions))
 
     async def catching_coro(
         self,
-        target: Coroutine,
-        exceptions: Optional[Union[Type[Exception], Tuple]] = None,
-    ) -> Any:
+        target: typing.Coroutine,
+        exceptions: type[Exception] | tuple | None = None,
+    ) -> typing.Any:
         """Wrap a target coro and catch specified exceptions."""
         if exceptions is None:
             exceptions = (asyncio.TimeoutError, ZigbeeException)
@@ -293,3 +319,22 @@ class CatchingTaskMixin(LocalLogMixin):
             self.exception("%s", exc_msg)
 
         return None
+
+
+def deprecated(message: str) -> typing.Callable[[typing.Callable], typing.Callable]:
+    """
+    Decorator that emits a DeprecationWarning when the function or property is accessed.
+    """
+
+    def decorator(function: typing.Callable) -> typing.Callable:
+        @functools.wraps(function)
+        def replacement(*args, **kwargs):
+            warnings.warn(
+                f"{function.__name__} is deprecated: {message}", DeprecationWarning
+            )
+
+            return function(*args, **kwargs)
+
+        return replacement
+
+    return decorator
