@@ -4,6 +4,7 @@ from __future__ import annotations
 
 import zigpy.types as t
 from zigpy.zcl import Cluster
+from zigpy.zcl.foundation import ZCLAttributeDef, ZCLCommandDef
 
 
 class DateTime(t.Struct):
@@ -14,30 +15,34 @@ class DateTime(t.Struct):
 class GenericTunnel(Cluster):
     cluster_id = 0x0600
     ep_attribute = "generic_tunnel"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0001: ("max_income_trans_size", t.uint16_t),
         0x0002: ("max_outgo_trans_size", t.uint16_t),
         0x0003: ("protocol_addr", t.LVBytes),
     }
-    server_commands = {0x0000: ("match_protocol_addr", (), False)}
-    client_commands = {
-        0x0000: ("match_protocol_addr_response", (), True),
-        0x0001: ("advertise_protocol_address", (), False),
+    server_commands: dict[int, ZCLCommandDef] = {
+        0x00: ZCLCommandDef("match_protocol_addr", {}, False)
+    }
+    client_commands: dict[int, ZCLCommandDef] = {
+        0x00: ZCLCommandDef("match_protocol_addr_response", {}, True),
+        0x01: ZCLCommandDef("advertise_protocol_address", {}, False),
     }
 
 
 class BacnetProtocolTunnel(Cluster):
     cluster_id = 0x0601
     ep_attribute = "bacnet_tunnel"
-    attributes = {}
-    server_commands = {0x0000: ("transfer_npdu", (t.LVBytes,), False)}
-    client_commands = {}
+    attributes: dict[int, ZCLAttributeDef] = {}
+    server_commands: dict[int, ZCLCommandDef] = {
+        0x00: ZCLCommandDef("transfer_npdu", {"npdu": t.LVBytes}, False)
+    }
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class AnalogInputRegular(Cluster):
     cluster_id = 0x0602
     ep_attribute = "bacnet_regular_analog_input"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0016: ("cov_increment", t.Single),
         0x001F: ("device_type", t.CharacterString),
         0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
@@ -46,14 +51,14 @@ class AnalogInputRegular(Cluster):
         0x0076: ("update_interval", t.uint8_t),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class AnalogInputExtended(Cluster):
     cluster_id = 0x0603
     ep_attribute = "bacnet_extended_analog_input"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0011: ("notification_class", t.uint16_t),
         0x0019: ("deadband", t.Single),
@@ -67,19 +72,19 @@ class AnalogInputExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array) # Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {
-        0x0000: ("transfer_apdu", (), False),
-        0x0001: ("connect_req", (), False),
-        0x0002: ("disconnect_req", (), False),
-        0x0003: ("connect_status_noti", (), False),
+    server_commands: dict[int, ZCLCommandDef] = {
+        0x00: ZCLCommandDef("transfer_apdu", {}, False),
+        0x01: ZCLCommandDef("connect_req", {}, False),
+        0x02: ZCLCommandDef("disconnect_req", {}, False),
+        0x03: ZCLCommandDef("connect_status_noti", {}, False),
     }
-    client_commands = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class AnalogOutputRegular(Cluster):
     cluster_id = 0x0604
     ep_attribute = "bacnet_regular_analog_output"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0016: ("cov_increment", t.Single),
         0x001F: ("device_type", t.CharacterString),
         0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
@@ -88,14 +93,14 @@ class AnalogOutputRegular(Cluster):
         0x0076: ("update_interval", t.uint8_t),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class AnalogOutputExtended(Cluster):
     cluster_id = 0x0605
     ep_attribute = "bacnet_extended_analog_output"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0011: ("notification_class", t.uint16_t),
         0x0019: ("deadband", t.Single),
@@ -109,28 +114,28 @@ class AnalogOutputExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array)# Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class AnalogValueRegular(Cluster):
     cluster_id = 0x0606
     ep_attribute = "bacnet_regular_analog_value"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0016: ("cov_increment", t.Single),
         0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class AnalogValueExtended(Cluster):
     cluster_id = 0x0607
     ep_attribute = "bacnet_extended_analog_value"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0011: ("notification_class", t.uint16_t),
         0x0019: ("deadband", t.Single),
@@ -144,14 +149,14 @@ class AnalogValueExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array),  # Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class BinaryInputRegular(Cluster):
     cluster_id = 0x0608
     ep_attribute = "bacnet_regular_binary_input"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x000F: ("change_of_state_count", t.uint32_t),
         0x0010: ("change_of_state_time", DateTime),
         0x001F: ("device_type", t.CharacterString),
@@ -163,14 +168,14 @@ class BinaryInputRegular(Cluster):
         0x0073: ("time_of_sc_reset", DateTime),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class BinaryInputExtended(Cluster):
     cluster_id = 0x0609
     ep_attribute = "bacnet_extended_binary_input"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0006: ("alarm_value", t.Bool),
         0x0011: ("notification_class", t.uint16_t),
@@ -181,14 +186,14 @@ class BinaryInputExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array),  # Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class BinaryOutputRegular(Cluster):
     cluster_id = 0x060A
     ep_attribute = "bacnet_regular_binary_output"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x000F: ("change_of_state_count", t.uint32_t),
         0x0010: ("change_of_state_time", DateTime),
         0x001F: ("device_type", t.CharacterString),
@@ -201,14 +206,14 @@ class BinaryOutputRegular(Cluster):
         0x0073: ("time_of_sc_reset", DateTime),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class BinaryOutputExtended(Cluster):
     cluster_id = 0x060B
     ep_attribute = "bacnet_extended_binary_output"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0011: ("notification_class", t.uint16_t),
         0x0023: ("event_enable", t.bitmap8),
@@ -218,14 +223,14 @@ class BinaryOutputExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array),  # Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class BinaryValueRegular(Cluster):
     cluster_id = 0x060C
     ep_attribute = "bacnet_regular_binary_value"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x000F: ("change_of_state_count", t.uint32_t),
         0x0010: ("change_of_state_time", DateTime),
         0x0021: ("elapsed_active_time", t.uint32_t),
@@ -236,14 +241,14 @@ class BinaryValueRegular(Cluster):
         0x0073: ("time_of_sc_reset", DateTime),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class BinaryValueExtended(Cluster):
     cluster_id = 0x060D
     ep_attribute = "bacnet_extended_binary_value"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0006: ("alarm_value", t.Bool),
         0x0011: ("notification_class", t.uint16_t),
@@ -254,28 +259,28 @@ class BinaryValueExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array),  # Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class MultistateInputRegular(Cluster):
     cluster_id = 0x060E
     ep_attribute = "bacnet_regular_multistate_input"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x001F: ("device_type", t.CharacterString),
         0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class MultistateInputExtended(Cluster):
     cluster_id = 0x060F
     ep_attribute = "bacnet_extended_multistate_input"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0006: ("alarm_value", t.uint16_t),
         0x0011: ("notification_class", t.uint16_t),
@@ -287,14 +292,14 @@ class MultistateInputExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array),  # Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class MultistateOutputRegular(Cluster):
     cluster_id = 0x0610
     ep_attribute = "bacnet_regular_multistate_output"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x001F: ("device_type", t.CharacterString),
         0x0028: ("feed_back_value", t.enum8),
         0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
@@ -302,14 +307,14 @@ class MultistateOutputRegular(Cluster):
         0x004F: ("object_type", t.enum16),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class MultistateOutputExtended(Cluster):
     cluster_id = 0x0611
     ep_attribute = "bacnet_extended_multistate_output"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0011: ("notification_class", t.uint16_t),
         0x0023: ("event_enable", t.bitmap8),
@@ -319,27 +324,27 @@ class MultistateOutputExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array),  # Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class MultistateValueRegular(Cluster):
     cluster_id = 0x0612
     ep_attribute = "bacnet_regular_multistate_value"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x00A8: ("profile_name", t.CharacterString),
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
 
 
 class MultistateValueExtended(Cluster):
     cluster_id = 0x0613
     ep_attribute = "bacnet_extended_multistate_value"
-    attributes = {
+    attributes: dict[int, ZCLAttributeDef] = {
         0x0000: ("acked_transitions", t.bitmap8),
         0x0006: ("alarm_value", t.uint16_t),
         0x0011: ("notification_class", t.uint16_t),
@@ -351,5 +356,5 @@ class MultistateValueExtended(Cluster):
         # 0x0082: ('event_time_stamps', TODO.array),  # Array[3] of (16-bit unsigned
         # integer, time of day, or structure of (date, time of day))
     }
-    server_commands = {}
-    client_commands = {}
+    server_commands: dict[int, ZCLCommandDef] = {}
+    client_commands: dict[int, ZCLCommandDef] = {}
